searchData={"items":[{"type":"module","title":"BenchmarkHelper","doc":"","ref":"BenchmarkHelper.html"},{"type":"macro","title":"BenchmarkHelper.inject_in_dev/0","doc":"","ref":"BenchmarkHelper.html#inject_in_dev/0"},{"type":"module","title":"DeltaCrdt","doc":"Start and interact with the Delta CRDTs provided by this library.\n\nA CRDT is a conflict-free replicated data-type. That is to say, it is a distributed data structure that automatically resolves conflicts in a way that is consistent across all replicas of the data. In other words, your distributed data is guaranteed to eventually converge globally.\n\nNormal CRDTs (otherwise called \"state CRDTs\") require transmission of the entire CRDT state with every change. This clearly doesn't scale, but there has been exciting research in the last few years into \"Delta CRDTs\", CRDTs that only transmit their deltas. This has enabled a whole new scale of applications for CRDTs, and it's also what this library is based on.\n\nA Delta CRDT is made of two parts. First, the data structure itself, and second, an anti-entropy algorithm, which is responsible for ensuring convergence. `DeltaCrdt` implements Algorithm 2 from [\"Delta State Replicated Data Types – Almeida et al. 2016\"](https://arxiv.org/pdf/1603.01529.pdf) which is an anti-entropy algorithm for δ-CRDTs. `DeltaCrdt` also implements join decomposition to ensure that deltas aren't transmitted unnecessarily in the cluster.\n\nWhile it is certainly interesting to have a look at this paper and spend time grokking it, in theory I've done the hard work so that you don't have to, and this library is the result.\n\nWith this library, you can build distributed applications that share some state. [`Horde.Supervisor`](https://hexdocs.pm/horde/Horde.Supervisor.html) and [`Horde.Registry`](https://hexdocs.pm/horde/Horde.Registry.html) are both built atop `DeltaCrdt`, but there are certainly many more possibilities.\n\nHere's a simple example for illustration:\n\n```\niex> {:ok, crdt1} = DeltaCrdt.start_link(DeltaCrdt.AWLWWMap, sync_interval: 3)\niex> {:ok, crdt2} = DeltaCrdt.start_link(DeltaCrdt.AWLWWMap, sync_interval: 3)\niex> DeltaCrdt.set_neighbours(crdt1, [crdt2])\niex> DeltaCrdt.set_neighbours(crdt2, [crdt1])\niex> DeltaCrdt.to_map(crdt1)\n%{}\niex> DeltaCrdt.put(crdt1, \"CRDT\", \"is magic!\")\niex> Process.sleep(10) # need to wait for propagation for the doctest\niex> DeltaCrdt.to_map(crdt2)\n%{\"CRDT\" => \"is magic!\"}\n```","ref":"DeltaCrdt.html"},{"type":"function","title":"DeltaCrdt.child_spec/1","doc":"Include DeltaCrdt in a supervision tree with `{DeltaCrdt, [crdt: DeltaCrdt.AWLWWMap, name: MyCRDTMap]}`","ref":"DeltaCrdt.html#child_spec/1"},{"type":"function","title":"DeltaCrdt.delete/3","doc":"","ref":"DeltaCrdt.html#delete/3"},{"type":"function","title":"DeltaCrdt.drop/3","doc":"","ref":"DeltaCrdt.html#drop/3"},{"type":"function","title":"DeltaCrdt.get/3","doc":"","ref":"DeltaCrdt.html#get/3"},{"type":"function","title":"DeltaCrdt.merge/3","doc":"","ref":"DeltaCrdt.html#merge/3"},{"type":"function","title":"DeltaCrdt.mutate/4","doc":"Mutate the CRDT synchronously.\n\nFor the asynchronous version of this function, see `mutate_async/3`.\n\nTo see which operations are available, see the documentation for the crdt module that was provided in `start_link/3`.\n\nFor example, `DeltaCrdt.AWLWWMap` has a function `add` that takes 4 arguments. The last 2 arguments are supplied by DeltaCrdt internally, so you have to provide only the first two arguments: `key` and `val`. That would look like this: `DeltaCrdt.mutate(crdt, :add, [\"CRDT\", \"is magic!\"])`. This pattern is repeated for all mutation functions. Another example: to call `DeltaCrdt.AWLWWMap.clear`, use `DeltaCrdt.mutate(crdt, :clear, [])`.","ref":"DeltaCrdt.html#mutate/4"},{"type":"function","title":"DeltaCrdt.mutate_async/3","doc":"Mutate the CRDT asynchronously.","ref":"DeltaCrdt.html#mutate_async/3"},{"type":"function","title":"DeltaCrdt.put/4","doc":"","ref":"DeltaCrdt.html#put/4"},{"type":"function","title":"DeltaCrdt.read/1","doc":"Read the state of the CRDT.\n\nForwards arguments to the used crdt module, so `read(crdt, [\"my-key\"])` would call `crdt_module.read(state, [\"my-key\"])`.\n\nFor example, `DeltaCrdt.AWLWWMap` accepts a list of keys to limit the returned values instead of returning everything.","ref":"DeltaCrdt.html#read/1"},{"type":"function","title":"DeltaCrdt.read/2","doc":"","ref":"DeltaCrdt.html#read/2"},{"type":"function","title":"DeltaCrdt.read/3","doc":"","ref":"DeltaCrdt.html#read/3"},{"type":"function","title":"DeltaCrdt.set_neighbours/2","doc":"Notify a CRDT of its neighbours.\n\nThis function allows CRDTs to communicate with each other and sync their states.\n\n**Note: this sets up a unidirectional sync, so if you want bidirectional syncing (which is normally desirable), then you must call this function twice (or thrice for 3 nodes, etc):**\n```\nDeltaCrdt.set_neighbours(c1, [c2, c3])\nDeltaCrdt.set_neighbours(c2, [c1, c3])\nDeltaCrdt.set_neighbours(c3, [c1, c2])\n```","ref":"DeltaCrdt.html#set_neighbours/2"},{"type":"function","title":"DeltaCrdt.start_link/2","doc":"Start a DeltaCrdt and link it to the calling process.\n\nThere are a number of options you can specify to tweak the behaviour of DeltaCrdt:\n- `:sync_interval` - the delta CRDT will attempt to sync its local changes with its neighbours at this interval (specified in milliseconds). Default is 200.\n- `:on_diffs` - function which will be invoked on every diff\n- `:max_sync_size` - maximum size of synchronization (specified in number of items to sync)\n- `:name` - name of the CRDT process\n- `:storage_module` - module which implements `DeltaCrdt.Storage` behaviour","ref":"DeltaCrdt.html#start_link/2"},{"type":"function","title":"DeltaCrdt.take/3","doc":"","ref":"DeltaCrdt.html#take/3"},{"type":"function","title":"DeltaCrdt.to_map/2","doc":"","ref":"DeltaCrdt.html#to_map/2"},{"type":"type","title":"DeltaCrdt.crdt_option/0","doc":"","ref":"DeltaCrdt.html#t:crdt_option/0"},{"type":"type","title":"DeltaCrdt.crdt_options/0","doc":"","ref":"DeltaCrdt.html#t:crdt_options/0"},{"type":"type","title":"DeltaCrdt.diff/0","doc":"","ref":"DeltaCrdt.html#t:diff/0"},{"type":"type","title":"DeltaCrdt.key/0","doc":"","ref":"DeltaCrdt.html#t:key/0"},{"type":"type","title":"DeltaCrdt.t/0","doc":"","ref":"DeltaCrdt.html#t:t/0"},{"type":"type","title":"DeltaCrdt.value/0","doc":"","ref":"DeltaCrdt.html#t:value/0"},{"type":"module","title":"DeltaCrdt.AWLWWMap","doc":"","ref":"DeltaCrdt.AWLWWMap.html"},{"type":"function","title":"DeltaCrdt.AWLWWMap.add/4","doc":"","ref":"DeltaCrdt.AWLWWMap.html#add/4"},{"type":"function","title":"DeltaCrdt.AWLWWMap.clear/2","doc":"","ref":"DeltaCrdt.AWLWWMap.html#clear/2"},{"type":"function","title":"DeltaCrdt.AWLWWMap.read/1","doc":"","ref":"DeltaCrdt.AWLWWMap.html#read/1"},{"type":"function","title":"DeltaCrdt.AWLWWMap.read/2","doc":"","ref":"DeltaCrdt.AWLWWMap.html#read/2"},{"type":"function","title":"DeltaCrdt.AWLWWMap.remove/3","doc":"","ref":"DeltaCrdt.AWLWWMap.html#remove/3"},{"type":"module","title":"DeltaCrdt.CausalCrdt.Diff","doc":"","ref":"DeltaCrdt.CausalCrdt.Diff.html"},{"type":"behaviour","title":"DeltaCrdt.Storage","doc":"This behaviour can be used to enable persistence of the CRDT.\n\nThis can be helpful in the event of crashes.\n\nTo use, implement this behaviour in a module, and pass it to your CRDT with the `storage_module` option.","ref":"DeltaCrdt.Storage.html"},{"type":"callback","title":"DeltaCrdt.Storage.read/1","doc":"","ref":"DeltaCrdt.Storage.html#c:read/1"},{"type":"callback","title":"DeltaCrdt.Storage.write/2","doc":"","ref":"DeltaCrdt.Storage.html#c:write/2"},{"type":"opaque","title":"DeltaCrdt.Storage.storage_format/0","doc":"","ref":"DeltaCrdt.Storage.html#t:storage_format/0"},{"type":"type","title":"DeltaCrdt.Storage.t/0","doc":"","ref":"DeltaCrdt.Storage.html#t:t/0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,49]}}